##' Compute Fibonacci numbers
##'
##' These function are wrappers to underlying C functions.

#-----------------------------------------------------------  
#dyn.load('../src/FibC1.so')

FibC0 <- function(n){
  ## Do some error checking
  if(n < 1) stop("n must be a positive integer!")

  # Handle trivial cases
  if (n==1) return(1L)
  if (n==2) return(c(1L,1L))

  ## create an R object to store result
  out=vector(length=n,mode="integer")

  # initialize the first two ones
  out[1:2] = 1L

  ## Note the coercion to ensure types match
  ## Also note how we return the desired value.
  .C("C_FibC0", as.integer(n), as.integer(out))[[2]]
}


#-----------------------------------------------------------
#dyn.load('../src/FibC1.so')

FibC1 <- function(n, last_only=TRUE){

  # Error handling
  n = as.integer(n) 
  if(n < 1 || is.na(n)) stop("n must be a positive integer!")

  # Handle n=1
  if (n==1) return(1L)

  # Check last_only and return
  if (last_only) {
    .C("C_FibC1_n", n, 0L)[[2]]
  } else {
    .C("C_FibC1", n, rep(1L, n))[[2]]
  }
 
}

#----------------------------------------------------------- 
#dyn.load('../src/FibC2.so')

FibC2 <- function(n){

  # Error handling 
  n = as.integer(n)
  if(n < 1 || is.na(n)) stop("n must be a positive integer!") 

  .C("C_FibC2", n, rep(1L, n))[[2]]
}


#-----------------------------------------------------------  
#dyn.load('../src/FibC3.so') 

FibC3 <- function(n){ 

 # Error handling 
 n = as.integer(n)
 if(n < 1 || is.na(n)) stop("n must be a positive integer!") 
 
 .C("C_FibC3", n, rep(1L, n))[[2]] 
}

#-----------------------------------------------------------  
#dyn.load('../src/RecFib.so')
FibC4 <- function(n, last_only=TRUE){

 # Error handling  
 n = as.integer(n)  
 if(n < 1 || is.na(n)) stop("n must be a positive integer!") 

 # Check last_only and return  
 if (last_only) {     
   .C("C_RecFib_one", n, 0L)[[2]]
 } else {
   .C("C_RecFib_all", n, rep(1L, n))[[2]]
 }

}

#----------------------------------------------------------- 
#dyn.load('../src/FibCall.so')

FibCall <- function(n){
  # Error handling 
  n = as.integer(n)  
  if(n < 1 || is.na(n)) stop("n must be a positive integer!")   

  .Call("C_FibCall", n)
}      